// UDA'Software 2002
// File: UDANetSend.cxx
// Version 1.6

/*

Copyright (C) 1988 Free Software Foundation, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*

Questo programma serve per inviare un messaggio ad un PC collegato alla rete
sfruttando i servizi offerti dal Sistema Operativo Windows NT 4.0.

In particolare, viene utilizzato il comando DOS:  NET SEND

******************************************************************************

Per problemi, ulteriori informazione o qualsiasi richiesta riguardante questo
software o comunque tutti quelli prodotti dalla UDA'Software e' possibile
contattare il Supporto Tecnico al seguente indirizzo E-Mail:

mailto:supporto_tecnico@uda.itgo.com

oppure tramite i servizi offerti direttamente dal sito WEB della UDA'Software:

http://uda.itgo.com

UDA'Software 2002, Copyright (C).

*/

// Librerie Standards
#include <string>
#include <vector>
#include <fstream.h>
// #include <stdlib.h>

// API di Windows
#include <windows.h>

// MessageDialog di FLTK
#include <FL/fl_ask.H>

// Header di UDA Net Send
#include "UNS.h"

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

// Define.
#define WINLABEL			"UDA Net Send Version 1.6 for Windows NT By UDA'Software ( http://digilander.libero.it/udasoft )"
#define STR_ERROR_BAD_PARAM		"Attenzione ! Devi riempire entrambi i campi di immissione testo opportunamente."
#define STR_ERROR_NOT_SEND		"Attenzione !! Messaggio NON spedito."
#define STR_ERROR_HELP_ERROR		"Attenzione !! E' risultato impossibile visualizzare la Guida di UDA Net Send."
#define STR_MESSAGE_NOERASE		"Nessun \"PC Name\" selezionato dalla lista; non e' stata intrapresa nessuna azione."
#define STR_MESSAGE_SENDED		"Messaggio spedito con successo."
#define STR_MESSAGE_DEL_PC_NAME		"Vuoi eliminare il Nome del PC selezionato dalla lista ?"
#define COMMAND				"NET SEND \""
#define DELETE_PC_STRING_COMMAND	"_deletepc"
#define DATFILE				".uns"
#define HELPFILE			"leggimi.txt"
#define MAXSERVERNAME			128
#define MAXMESSAGE			1024

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

// Variabili Globali.
Fl_Window		*window;
typedef vector<char*>	vString;
vString			*pvsListPCName;
int			iFlagExit;

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

int main (int argc, char **argv) {
  int		iNumPCNameAdded = 1;		// Numero di Nomi di PC aggiunti alla List-box.

  // Imposta le variabili globali ai valori di default.
  iFlagExit = 0;

  // Se il programma e' stato lanciato con un parametro "qualsiasi",
  // evita di uscire dopo aver eseguito un operazione.
  if (argc >= 2) {
    if (!strcasecmp(argv[1], "noclose"))
      iFlagExit = 1;
  }

  // Crea l'oggetto finestra.
  window = make_window();

  // La visualizza.
  window->end();
  window->show();

  // Modifica il Caption.
  window->label(WINLABEL);

  // Alloca memoria per il vettore contenente la lista dei PC ai quali e' stato inviato un messaggio.
  pvsListPCName = new vString();

  // Apre l'eventuale files contenente la lista dei PCs.
  ifstream	*pfsPCNameFile = new ifstream(DATFILE, ios::in);

  // Se il file esiste ed e' stato aperto provvede ad immagazzinare i nomi dei PCs ai quali e' stato
  // inviato un messaggio, altrimenti, disattiva la List-box che dovrebbe contenerli.
  if (!(*pfsPCNameFile)) {
    UDANetSebd_ChoicePC->deactivate();
  } else {
    char		*psBuffer;
    while(!pfsPCNameFile->eof()) {
      psBuffer = new char[MAXSERVERNAME];
      psBuffer[0] = 0;
      pfsPCNameFile->read(psBuffer, MAXSERVERNAME);
      psBuffer[MAXSERVERNAME - 2] = 0;

      if (strlen(psBuffer)) {
	// Aggiunge il nome del PC letto nella list-box.
	UDANetSebd_ChoicePC->add(psBuffer);

	// Se l'aggiunta e' effettivamente avventua, procede all'inserimento anche nel vettore dinamico.
	if (iNumPCNameAdded < int(UDANetSebd_ChoicePC->size())) {
	  pvsListPCName->push_back(psBuffer);
	  iNumPCNameAdded++;
	}
      }
    }
  }

  // Se non ci sono nomi di PC nel vettore visualizza la guida.
  if (!pvsListPCName->size()) {
    CmdUDANetSendButtonHelp(0, 0);
  }

  // Chiude il file dati ed elimina il puntatore.
  pfsPCNameFile->close();
  delete pfsPCNameFile;

  // Lancia il gestore degli eventi.
  return Fl::run();
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

void
CmdUDANetSendButtonHelp(Fl_Button *, void *) {
  // Crea l'oggetto finestra.
  Fl_Window	*winHelp = make_winHelp();

  // La visualizza.
  winHelp->end();
  winHelp->show();

  // Visualizza la guida.
  if (!UDANetSend_BrowserHelp->load(HELPFILE)) {
    winHelp->hide();
    fl_alert(STR_ERROR_HELP_ERROR);
  }
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

void
CmdUDANetSendTextInputPC(Fl_Input *, void *) {
  // Cattura il focus del campo immissione testi dove inserire il messaggio.
  UDANetSend_TextInputMessage->take_focus();
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

void
CmdUDANetSendChoicePC(Fl_Choice *, void *) {
  // Imposta il Nome del PC al quale dovra' essere spedito il messaggio.
  UDANetSend_TextInputPC->value((*pvsListPCName)[UDANetSebd_ChoicePC->value()]);

  // Cattura il focus del campo immissione testi dove inserire il messaggio.
  UDANetSend_TextInputMessage->take_focus();
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// __________________________________________________________________________________________________________________________

void
CmdUDANetSendTextInputMessage(Fl_Input *, void *) {
  char	*psServerName = new char[MAXSERVERNAME];						// Il Nome del Server al quale inviare il messaggio.
  char	*psMessage = new char[MAXMESSAGE];							// Il Messaggio.
  char	*psCommand = new char[MAXMESSAGE + (MAXSERVERNAME * 2)];		// Il comando da lanciare al Sistema Operativo.

  // Verifica se i dati immessi negli appositi campi di immissione testo (Server e Messaggio) sono corretti.
  if (strlen(UDANetSend_TextInputPC->value()) && strlen(UDANetSend_TextInputMessage->value())) {
    // Copia i valori dei campi immissione testo nelle rispettive variabili.
    strncpy(psServerName, UDANetSend_TextInputPC->value(), MAXSERVERNAME - 2);
    psServerName[MAXSERVERNAME - 1] = 0;
    strncpy(psMessage, UDANetSend_TextInputMessage->value(), MAXMESSAGE - 2);
    psMessage[MAXMESSAGE - 1] = 0;

    // Compone il Comando Completo da lanciare al Sistema Operativo.
    strcpy(psCommand, 
	   string(string(COMMAND) + 
		  string(psServerName) + 
		  string("\" \"") + 
		  string(psMessage) +
		  string("\"")).c_str());
    
    if (string(psMessage) != string(DELETE_PC_STRING_COMMAND)) {
      // Lancia il comando al Sistema Operativo.
      WinExec(psCommand, FALSE);

      // Inserisce il nome del pc nella lista.
      pvsListPCName->push_back(psServerName);

      // Avverte l'utente che il messaggio e' stato spedito correttamente.
      fl_alert(STR_MESSAGE_SENDED);
    } else {
      // Se il messaggio e' vuoto, chiede all'utente se desidera eliminare dalla lista il nome del PC.
      if (pvsListPCName->size() && UDANetSebd_ChoicePC->value() >= 0 && fl_ask(STR_MESSAGE_DEL_PC_NAME)) {
	pvsListPCName->erase(pvsListPCName->begin() + UDANetSebd_ChoicePC->value());
      } else {
	fl_alert(STR_MESSAGE_NOERASE);
      }
    }

    if (!iFlagExit) {
      // Nasconde la finestra.
      window->hide();
    }
  } else {
    // Se i campi di immissione testo risultano vuoti, comunica opportuno messaggio di errore.
    fl_alert(STR_ERROR_BAD_PARAM);
    fl_alert(STR_ERROR_NOT_SEND);

    // Dealloca la memoria delle stringhe dinamiche.
    delete [] psServerName;
    delete [] psMessage;
    delete [] psCommand;

    // Esce con errore.
    if (!iFlagExit) {
      exit(1);
    }
  }

  // Salva su disco la Lista dei PC ai quali e' stato spedito un messaggio.
  // Crea il puntatore al file di output e lo apre.
  ofstream	*pfsPCNameFile = new ofstream(DATFILE, ios::out);

  if (int(pvsListPCName->size())) {
    for (int i = 0; i < int(pvsListPCName->size()); i++) {
      pfsPCNameFile->write((*pvsListPCName)[i], MAXSERVERNAME);
    }
  } else {
    if (string(psMessage) != string(DELETE_PC_STRING_COMMAND)) {
      pfsPCNameFile->write(psServerName, MAXSERVERNAME);
    }
  }

  // Chiude il file dati ed elimina il puntatore.
  pfsPCNameFile->close();
  delete pfsPCNameFile;

  // Dealloca la memoria delle stringhe dinamiche.
  delete [] psServerName;
  delete [] psMessage;
  delete [] psCommand;

  // Esce con esito positivo.
  if (!iFlagExit) {
    exit(0);
  }
}
